// **** BEGIN LICENSE BLOCK ****
//   Copyright(c) 2009-2010 MATSUZAWA Jiro
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
//   02110-1301, USA.  */
// **** END LICENSE BLOCK ****

var rencon;
if (!rencon) {
    rencon = {};
} else if (typeof rencon != "object") {
    throw new Error("cannot use rencon");
}

rencon.CiNiiSearchParser = function()
{
}

rencon.CiNiiSearchParser.getInstance = function()
{
    if (rencon.CiNiiSearchParser.instance == null) {
        rencon.CiNiiSearchParser.instance = new rencon.CiNiiSearchParser();
    }
    return rencon.CiNiiSearchParser.instance;
}


//////////////////////////
// class property
//////////////////////////

rencon.CiNiiSearchParser.instance = null;


//////////////////////////
// prototype
//////////////////////////

rencon.CiNiiSearchParser.prototype = {
    pickoutBibRecord : function()
    {
        var recordList;
        var record;
        var title;
        var creator;
        var journal;
        var volume;
        var pubDate;
        var pages;
        var url;

        // Exception is transmitted
        recordList = this._getRecordList();

        for (var i = 0; i < recordList.length; ++i) {
            title = this._getTitle(recordList[i]);
            creator = this._getCreator(recordList[i]);
            journal = this._getJournal(recordList[i]);
            volume = this._getVolume(recordList[i]);
            pubDate = this._getPubDate(recordList[i]);
            pages = this._getPages(recordList[i]);
            url = this._getUrl(recordList[i]);

            record = new rencon.ArticleRecord(title, creator, pubDate, url, journal, volume, pages);
            rencon.bibTable.addRecord(record);
        }
    },


    _getRecordList : function()
    {
        var recordList = null;
        var recordList = window._content.document.getElementById("itemlistbox").getElementsByTagName("li");
        if (!recordList) {
            throw new Error("no records information");
        }

        return recordList;
    },


    _getTitle : function(
            item
            )
    {
        var titleNodes = item.getElementsByTagName("a")[0].childNodes;
        var title = "";

        for (var i = 0; i < titleNodes.length; ++i) {
            title += this._extractText(titleNodes[i]);
        }

        return title;
    },


    _getCreator : function(
            item
            )
    {
        var creator = "";
        var nodes = item.getElementsByClassName("item_authordata")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            creator += this._extractText(nodes[i]);
        }
        creator = creator.replace(/\s{2,}/g, "");
        return creator.replace(/,/g, ", ");
    },


    _getJournal : function(
            item
            )
    {
        var journal = "";
        var nodes = item.getElementsByClassName("journal_title")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            journal += this._extractText(nodes[i]);
        }
        return journal;
    },


    _getVolume : function(
            item
            )
    {
        var volume = "";
        var tsugo = "";
        var nodes = item.getElementsByClassName("volume")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            volume += this._extractText(nodes[i]);
        }
        tsugo = this._getTsugo(item);
        if (tsugo != "") {
            volume = volume + " " + tsugo;
        }
        volume = this._deleteHeadSpace(volume);
        volume = this._deleteTailComma(volume);
        return volume;
    },


    _getPubDate : function(
            item
            )
    {
        var pubDate = "";
        var nodes = item.getElementsByClassName("report")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            pubDate += this._extractText(nodes[i]);
        }
        pubDate = this._deleteHeadComma(pubDate);
        return pubDate;
    },


    _getPages : function(
            item
            )
    {
        var pages = "";
        var nodes = item.getElementsByClassName("pages")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            pages += this._extractText(nodes[i]);
        }
        return pages;
    },


    _getUrl : function(
            item
            )
    {
        var pdfList = item.getElementsByClassName("bodypdf");
        if (pdfList[0]) {
            return pdfList[0].href;
        }
        return "";
    },


    _getTsugo : function(
            item
            )
    {
        var tsugo = "";
        var nodes = item.getElementsByClassName("tsugo")[0].childNodes;
        for (var i = 0; i < nodes.length; ++i) {
            tsugo += this._extractText(nodes[i]);
        }
        return tsugo;
    },


    _extractText : function(
            node
            )
    {
        var text;
        if (node instanceof Text) {
            text = this._trimText(node.data);
            return text;
        } else if (node instanceof HTMLSpanElement) {
            text = node.textContent;
            return text;
        }
        return "";
    },

    _trimText : function(
            text
            )
    {
        var result;
        if (typeof text != "string") {
            return "";
        }
        result = text.replace(/^\s{2,}/, "");
        result = result.replace(/\s{2,}$/, "");

        return result;
    },


    _deleteTailComma : function(
            text
            )
    {
        var lastIdx;

        if (typeof text != "string") {
            return "";
        }

        lastIdx = text.length - 1;
        if (text[lastIdx] == ",") {
            return text.slice(0, lastIdx);
        }
        return text;
    },


    _deleteHeadComma : function(
            text
            )
    {
        if (typeof text != "string") {
            return "";
        }

        if (text[0] == ",") {
            return text.slice(1, text.length);
        }
        return text;
    },


    _deleteHeadSpace : function(
            text
            )
    {
        if (typeof text != "string") {
            return "";
        }

        if (text[0] == " ") {
            return text.slice(1, text.length);
        }
        return text;
    },
};
